#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013, 2014 CERN
# Author: Pawel Szostek (pawel.szostek@cern.ch)
# Modified to allow ISim simulation by Lucas Russo (lucas.russo@lnls.br)
# Multi-tool support by Javier D. Garcia-Lasheras (javier@garcialasheras.com)
#
# This file is part of Hdlmake.
#
# Hdlmake is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hdlmake is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Hdlmake.  If not, see <http://www.gnu.org/licenses/>.

"""Module providing the core functionality for writing Makefiles"""

import os

from hdlmake.util import path as path_mod


class ToolMakefile(object):

    """Class that provides the Makefile writing methods and status"""

    def __init__(self, filename=None):
        super(ToolMakefile, self).__init__()
        self._file = None
        self._initialized = False
        self._tool_info = {}
        self._clean_targets = {}
        self._tcl_controls = {}
        self._hdl_files = []
        self._supported_files = []
        self.top_module = None
        self.fileset = None
        if filename:
            self._filename = filename
        else:
            self._filename = "Makefile"

    def __del__(self):
        if self._file:
            self._file.close()

    def makefile_setup(self, top_module, fileset):
        """Set the Makefile configuration"""
        self.top_module = top_module
        self.fileset = fileset

    def makefile_includes(self):
        """Add the included makefiles that need to be previously loaded"""
        for file_aux in self.top_module.incl_makefiles:
            if os.path.exists(file_aux):
                self.write("include %s\n" % file_aux)

    def makefile_clean(self):
        """Print the Makefile target for cleaning intermediate files"""
        self.writeln("#target for cleaning intermediate files")
        self.writeln("clean:")
        tmp = "\t\t" + path_mod.del_command() + \
            " $(LIBS) " + ' '.join(self._clean_targets["clean"])
        self.writeln(tmp)

    def makefile_mrproper(self):
        """Print the Makefile target for cleaning final files"""
        self.writeln("#target for cleaning final files")
        self.writeln("mrproper: clean")
        tmp = "\t\t" + path_mod.del_command() + \
            " " + ' '.join(self._clean_targets["mrproper"])
        self.writeln(tmp)

    def initialize(self):
        """Open the Makefile file and print a header if not initialized"""
        if not self._initialized:
            if os.path.exists(self._filename):
                if os.path.isfile(self._filename):
                    os.remove(self._filename)
                elif os.path.isdir(self._filename):
                    os.rmdir(self._filename)

            self._file = open(self._filename, "a+")
            self._initialized = True
            self.writeln("########################################")
            self.writeln("#  This file was generated by hdlmake  #")
            self.writeln("#  http://ohwr.org/projects/hdl-make/  #")
            self.writeln("########################################")
            self.writeln()
        elif not self._file:
            self._file = open(self._filename, "a+")

    def write(self, line=None):
        """Write a string in the manifest, no new line"""
        if not self._initialized:
            self.initialize()
        self._file.write(line)

    def writeln(self, text=None):
        """Write a string in the manifest, automatically add new line"""
        if text is None:
            self.write("\n")
        else:
            self.write(text + "\n")
